@@ -7,7 +7,7 @@
 	2014.01.22 V00.02 Bug fixed for beep.c
 	2014.01.23 V00.03 drop "htons"
 	2014.01.23 V00.04 Added RnRoute messages routine
-
+	2014.01.28 V00.05 Changed "hole punching" routine & bug fixed
 */
 
 #include <stdio.h>
@@ -49,16 +49,20 @@ in_port_t	dstPort;
 
 struct	mq_attr	rf_mattr;
 struct	mq_attr	inet_mattr;
+struct  repMsg  reqConMsg;
 
 pthread_t	usb_ID;
 
 char	SendConnect;
 char	RecvConnect;
-struct  packet  send_pkt;
+struct  packet  	send_pkt;
 struct	shm_form	send_msg;
 int	AckTimeOut;
+int	DSTREQcnt;
 char	RelaySW;
 char	respFlags;
+char	Connected;
+char	DVwait;
 
 char	PTT_On;
 char	COS_On;
@@ -96,7 +100,7 @@ void	send_dsvt(void)
 			memcpy ((char *)&RelayPacket.dv, (char *)&send_pkt, 56);
 			RelayPacket.ip = dstIP;
 			RelayPacket.port = dstPort;
-        		sndAddr.sin_family = AF_INET;
+        		sndAddr.sin_family = PF_INET;
         		sndAddr.sin_port = trust_db_port;
         		sndAddr.sin_addr = dbIP;
         		sendto (udpSocket, (char *)&RelayPacket, sizeof(RelayPacket), 0,
@@ -106,7 +110,7 @@ void	send_dsvt(void)
 		{
                 	sndAddr.sin_addr = dstIP;
                 	sndAddr.sin_port = dstPort;
-                	sndAddr.sin_family = AF_INET;
+                	sndAddr.sin_family = PF_INET;
                 	sendto (udpSocket, &send_pkt, 56, 0,
                         	(struct sockaddr *)&sndAddr, sizeof(sndAddr));
 		}
@@ -114,13 +118,16 @@ void	send_dsvt(void)
 	}
 	length = mq_receive (rf_qid, (char *)&send_pkt, sizeof(send_pkt), NULL); 
 	if (length > 0)
-	{	
+	{
 		if (memcmp (send_pkt.ID, STREAM_ID, 4) == 0)
 		{
 			if (send_pkt.B_header.seq == 0x80)
 			{
 				ConSend();
-				ConnectWait = TRUE;	
+				ConnectWait = TRUE;
+				AckTimeOut = 0;
+				DSTREQcnt = 0;
+				Connected = FALSE;
 				return;
 			}
 			if (RelaySW)
@@ -128,7 +135,7 @@ void	send_dsvt(void)
                         	memcpy ((char *)&RelayPacket.dv, (char *)&send_pkt, 27);
                         	RelayPacket.ip = dstIP;
                         	RelayPacket.port = dstPort;
-                        	sndAddr.sin_family = AF_INET;
+                        	sndAddr.sin_family = PF_INET;
                         	sndAddr.sin_port = trust_db_port;
                         	sndAddr.sin_addr = dbIP;
                         	sendto (udpSocket, (char *)&RelayPacket, 33, 0,
@@ -137,8 +144,8 @@ void	send_dsvt(void)
 			else
 			{
                 		sndAddr.sin_addr = dstIP;
-                		sndAddr.sin_port = htons(dstPort);
-                		sndAddr.sin_family = AF_INET;
+                		sndAddr.sin_port = dstPort;
+                		sndAddr.sin_family = PF_INET;
         			sendto (udpSocket, &send_pkt, 27, 0,
                				(struct sockaddr *)&sndAddr, sizeof(sndAddr));
 			}
@@ -147,6 +154,7 @@ void	send_dsvt(void)
 				ConnectWait = FALSE;
 				FirstHeaderSend = FALSE;
 				RelaySW = FALSE;
+				Connected = FALSE;
 			}
 		}
 	}
@@ -160,49 +168,90 @@ void	receive_dsvt(void)
 	addrlen = sizeof(struct sockaddr_in);
 	recvfrom (udpSocket, (char *)&inet_recv, sizeof(inet_recv), 0, 
 		(struct sockaddr *)&sndAddr, &addrlen);
-        if (memcmp ((char *)&inet_recv, "\0\0\0\0SRCREQ ", 10) == 0)
+        if (memcmp ((char *)&inet_recv, "\0\0\0\0SRCREQ", 10) == 0)
         {
 		sendto (udpSocket, "\0\0\0\0SRCACK UDP hole punching", 28, 0,
-                                (struct sockaddr *)&sndAddr, sizeof(sndAddr));
+               		(struct sockaddr *)&sndAddr, sizeof(sndAddr));
         }
-        if (memcmp ((char *)&inet_recv, "\0\0\0\0DSTREQ ", 10) == 0)
+        if (memcmp ((char *)&inet_recv, "\0\0\0\0DSTREQ", 10) == 0)
         {
-                sendto (udpSocket, "\0\0\0\0DSTACK UDP hole punching", 28, 0,
-                        (struct sockaddr *)&sndAddr, sizeof(sndAddr));
+		if (!Connected)
+		{	
+                	ConnectWait = FALSE;
+                	FirstHeaderSend = TRUE;
+			dstPort = sndAddr.sin_port;
+			dstIP = sndAddr.sin_addr;
+			Connected = TRUE;
+		}
         }
-        if (memcmp ((char *)&inet_recv, "\0\0\0\0SRCACK ", 10) == 0)
+        if (memcmp ((char *)&inet_recv, "\0\0\0\0SRCACK", 10) == 0)
         {
-		ConnectWait = FALSE;
-		FirstHeaderSend = TRUE;
+		if (!Connected)
+		{
+			dstPort = sndAddr.sin_port;
+			dstIP = sndAddr.sin_addr;
+			ConnectWait = FALSE;
+			FirstHeaderSend = TRUE;
+			Connected = TRUE;
+		}
         }
 
 	if (memcmp ((char *)&inet_recv.ID, STREAM_ID, 4) == 0)
+	{
 		mq_send (inet_qid, (char *)&inet_recv, sizeof(inet_recv), 1);
+		DVwait = FALSE;
+	}
 }
 
-void    AckSend(char srcRptCall[8])
+void	AckSendTrust(char srcRptCall[8])
 {
         struct  shm_form        ack_msg;
-        struct  hostent *hp;
 
-        if (COS_On || PTT_On)	ack_msg.type = Nak;
-	else 	ack_msg.type = Ack;
+	ack_msg.type = Ack;
         memcpy (ack_msg.Rep1Call, srcRptCall, 8);
         memcpy (ack_msg.dstCall, client_call, 8);
-        sndAddr.sin_family = AF_INET;
+        sndAddr.sin_family = PF_INET;
         sndAddr.sin_port = trust_db_port;
 	sndAddr.sin_addr = dbIP;
         sendto (udpSocket, (char *)&ack_msg, sizeof(ack_msg), 0,
                  (struct sockaddr *)&sndAddr, sizeof(sndAddr));
+	return;
+}
+
+void    NakSendTrust(char srcRptCall[8])
+{
+        struct  shm_form        ack_msg;
+
+        ack_msg.type = Nak;
+        memcpy (ack_msg.Rep1Call, srcRptCall, 8);
+        memcpy (ack_msg.dstCall, client_call, 8);
+        sndAddr.sin_family = PF_INET;
+        sndAddr.sin_port = trust_db_port;
+        sndAddr.sin_addr = dbIP;
+        sendto (udpSocket, (char *)&ack_msg, sizeof(ack_msg), 0,
+                 (struct sockaddr *)&sndAddr, sizeof(sndAddr));
+        return;
+}
+
+void	ReqSendSender()
+{
+	sndAddr.sin_family = PF_INET;
+	sndAddr.sin_port = dstPort;
+	sndAddr.sin_addr = dstIP;
+	sendto (udpSocket, "\0\0\0\0DSTREQ UDP hole punching", 28, 0,
+		(struct sockaddr *)&sndAddr, sizeof(sndAddr));
 }
 
 void	ConSend(void)
 {
-	memcpy (&send_msg.Rep2Call, &send_pkt.header.Rpt2Call, 32);
+        memcpy (&send_msg.Rep2Call, &send_pkt.header.Rpt2Call, 32);
 	memcpy (&send_msg.dstRep, &send_pkt.header.YourCall, 8);
+
 	send_msg.type = ReqConnect;
 	send_msg.protocol = G3;
-	sndAddr.sin_family = AF_INET;
+	send_msg.PacketID[0] = send_pkt.B_header.PacketID[0];
+        send_msg.PacketID[1] = send_pkt.B_header.PacketID[1];
+	sndAddr.sin_family = PF_INET;
 	sndAddr.sin_port = trust_db_port;
 	sndAddr.sin_addr = dbIP;
 	sendto (udpSocket, (char *)&send_msg, sizeof (struct shm_form), 0,
@@ -220,7 +269,6 @@ int main(int argc, const char **argv)
 
 	// IP アドレス，ソケット，sockaddr_in 構造体
 	struct sockaddr_in dstAddr;
-	struct	repMsg	reqConMsg;
 
 	//struct sockaddr_in addr;
  	struct	hostent *hp;
@@ -256,6 +304,9 @@ int main(int argc, const char **argv)
 	respFlags = RespNULL;
 	repeater_mode = 0;
 	COS_On = FALSE;
+	Connected = FALSE;
+	DSTREQcnt = 0;
+	DVwait = FALSE;
 
 	config (argc, (char **)argv);
 
@@ -307,13 +358,13 @@ printf ("debug repeater mode %d\n",repeater_mode);
 
 	//sockaddr_in 構造体のセット
 	bzero((char *)&dstAddr, sizeof(dstAddr));
-	dstAddr.sin_family = AF_INET;
+	dstAddr.sin_family = PF_INET;
 	dstAddr.sin_port = trust_port;
 	dstAddr.sin_addr = trustIP;
 
 	//ソケットの生成
-	dstSocket = socket(AF_INET, SOCK_STREAM, 0);
-	udpSocket = socket(AF_INET, SOCK_DGRAM, 0);
+	dstSocket = socket(PF_INET, SOCK_STREAM, 0);
+	udpSocket = socket(PF_INET, SOCK_DGRAM, 0);
 
 	//接続
 	if (connect(dstSocket, (struct sockaddr *)&dstAddr, 
@@ -365,7 +416,7 @@ printf ("debug repeater mode %d\n",repeater_mode);
 	//ポート番号の送信
         send (dstSocket, buf, strlen(buf), 0);
 
-        udpAddr.sin_family = AF_INET;
+        udpAddr.sin_family = PF_INET;
         udpAddr.sin_port = htons(client_port);
         udpAddr.sin_addr.s_addr = htonl(INADDR_ANY);
         ret = bind(udpSocket, (struct sockaddr *)&udpAddr, sizeof(udpAddr));
@@ -394,19 +445,28 @@ printf ("debug repeater mode %d\n",repeater_mode);
 				if (ConnectWait)
 				{
 					AckTimeOut++;
-					if (AckTimeOut == 20)	/* 100 m sec, */
+					if (AckTimeOut == 40)	/* 100 m sec, */
 					{
 						ConSend();
 					}
-					if (AckTimeOut > 40)	/* 200 m sec. */
+					if (AckTimeOut > 80)	/* 200 m sec. */
 					{
 						RelaySW = TRUE;
 						ConnectWait = FALSE;
 						FirstHeaderSend = TRUE;
 						AckTimeOut = 0;
+						Connected = TRUE;
 						printf ("Change to Relay Mode\n");
 					}
 				}
+				if (DVwait)
+				{
+					DSTREQcnt++;
+					if ((DSTREQcnt % 4) == 0)	/* 20 m sec. interval */
+					{
+						ReqSendSender();
+					}
+				}
 				break;
 			default:
                                 if (FD_ISSET(udpSocket, &readOk))
@@ -426,41 +486,45 @@ printf ("debug repeater mode %d\n",repeater_mode);
                                         }
                         		if (reqConMsg.reqType == Ack)
                         		{
-                                		printf ("%2.2x %8.8s %s %d\n", 
-							reqConMsg.reqType, reqConMsg.ackConnect.YourCall, 
-							inet_ntoa (reqConMsg.ackConnect.ip),
-							reqConMsg.ackConnect.port);
-						dstPort = reqConMsg.ackConnect.port;
-						dstIP = reqConMsg.ackConnect.ip;
-                        			sndAddr.sin_family = AF_INET;
-                        			sndAddr.sin_port = htons(dstPort);
-                        			sndAddr.sin_addr = dstIP;
-						respFlags = RespNULL;
-                        			ret = sendto (udpSocket, "\0\0\0\0SRCREQ UDP hole punching", 28, 0,
-                                			(struct sockaddr *)&sndAddr, sizeof(sndAddr));
+						if (!Connected)
+						{
+							dstPort = reqConMsg.ackConnect.port;
+							dstIP = reqConMsg.ackConnect.ip;
+                        				sndAddr.sin_family = PF_INET;
+                        				sndAddr.sin_port = dstPort;
+                        				sndAddr.sin_addr = dstIP;
+							respFlags = RespNULL;
+                        				sendto (udpSocket, "\0\0\0\0SRCREQ UDP hole punching", 28, 0,
+                                				(struct sockaddr *)&sndAddr, sizeof(sndAddr));
+						}
                         		}
-                                        if (reqConMsg.reqType == Nak)
+                                        else if (reqConMsg.reqType == Nak)
                                         {
-                                                printf ("%2.2x %8.8s %s %d\n",
+                                                printf ("Nak %2.2x %8.8s %s %d\n",
                                                         reqConMsg.reqType, reqConMsg.ackConnect.YourCall,
                                                         inet_ntoa (reqConMsg.ackConnect.ip),
-                                                        reqConMsg.ackConnect.port);
+                                                        ntohs(reqConMsg.ackConnect.port));
 							respFlags = reqConMsg.ackConnect.RespFlags;
                                         }
                         		else if (reqConMsg.reqType == ReqConnect)
                         		{
-                                		printf ("%2.2x %2.2x %8.8s %s %d\n", 
+                                		printf ("Req %2.2x %2.2x %8.8s %s %d\n", 
 							reqConMsg.reqType,
 							reqConMsg.reqConnect.protocol, 
 							reqConMsg.reqConnect.YourCall, 
 							inet_ntoa(reqConMsg.reqConnect.ip),
-							reqConMsg.reqConnect.port);
-						AckSend (reqConMsg.reqConnect.Rpt1Call);
-                                                sndAddr.sin_family = AF_INET;
-                                                sndAddr.sin_port = htons(reqConMsg.reqConnect.port);
-                                                sndAddr.sin_addr = reqConMsg.reqConnect.ip;
-                                                sendto (udpSocket, "\0\0\0\0DSTREQ UDP hole punching", 28, 0,
-                                                        (struct sockaddr *)&sndAddr, sizeof(sndAddr));
+							ntohs(reqConMsg.reqConnect.port));
+						if (PTT_On || COS_On)
+						{
+							NakSendTrust(reqConMsg.reqConnect.Rpt1Call);
+						}
+						else
+						{
+							ReqSendSender();
+							DVwait = TRUE;
+							DSTREQcnt = 0;
+							AckSendTrust (reqConMsg.reqConnect.Rpt1Call);
+						}
                         		}
 				}
 				if (FD_ISSET(rf_qid, &readOk))
