@@ -93,7 +93,7 @@ void	from_rig(void)
 				if (!memcmp(LastFrameCheckRx, LastFrame0, 6))
 				{
 					dv_frame.B_header.seq = Packet_Seq | 0x40;
-					if (SendToInet) send_voice_queue();
+					send_voice_queue();
 					Packet_Seq++;
 					if (Packet_Seq > 20) Packet_Seq = 0;
 					headerRead = FALSE;
@@ -131,12 +131,11 @@ void	from_rig(void)
 						SendToInet = FALSE;
 						SendHeader = FALSE;	/* header sending switch */
 					}
-					if (SendToInet) send_voice_queue();
+					send_voice_queue();
 					Packet_Seq++;
 					if (Packet_Seq > 20) Packet_Seq = 0;
 					voice_pnt = 0;
 				}
-
 			}
 		}
 		else if (length == 0)
@@ -166,7 +165,6 @@ void	from_rig(void)
 
 void	send_voice_queue(void) 	/* read from rig (DV packet) */
 {
-	int	ret;
 	extern	char voice[12];
 	if (SendHeader)
 	{
@@ -189,12 +187,15 @@ void	send_voice_queue(void) 	/* read from rig (DV packet) */
 	dv_frame.B_header.PacketID[0] = (Packet_ID >> 8) & 0xff;
 	dv_frame.B_header.PacketID[1] = Packet_ID & 0xff;
 	memcpy ((char *)dv_frame.voice.voice, (char *)voice, 12);
-	ret = mq_send (rf_qid, (char *)&dv_frame, sizeof(dv_frame), 1);
+	if (SendToInet) mq_send (rf_qid, (char *)&dv_frame, sizeof(dv_frame), 1);
+	if (repeater_mode)
+	{
+		mq_send (inet_qid, (char *)&dv_frame, sizeof(dv_frame), 1);
+	}
 }
 
 void	send_header_queue(void) 	/* read from rig (DV packet) */
 {
-	int ret;
 
 		/* RF header send */
 		memcpy ((char *)dv_frame.ID,  "DSVT" ,4);
@@ -214,7 +215,13 @@ void	send_header_queue(void) 	/* read from rig (DV packet) */
 
 		memcpy ((char *)&dv_frame.header, (char *)&rf_header, 41);
 		memcpy ((char *)&dv_frame.header.Rpt1Call, client_call, 8);
-		ret = mq_send (rf_qid, (char *)&dv_frame, sizeof(dv_frame), 1);
+		if (SendToInet) mq_send (rf_qid, (char *)&dv_frame, sizeof(dv_frame), 1);
+
+		if (repeater_mode)
+		{
+			dv_frame.header.flags[0] = 0x00;
+			mq_send (inet_qid, (char *)&dv_frame, sizeof(dv_frame), 1);
+		}
 }
 
 int	BitsCheck(char pattern1[], char pattern2[], int cnt)
