@@ -1,3 +1,7 @@
+/*
+	Auther		Satoshi Yasuda	7m3tjz/ad6gz
+*/
+
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
